==> download-file?path=joomla%252Finvoicespro330%252Fplugins%252Fplg_invoicemanager_payplans_2.0.1_2.5.zip: invoicemanager.php <==
<?php
/*------------------------------------------------------------------------
# com_invoices - Invoice Manager for Joomla
# ------------------------------------------------------------------------
# author				Germinal Camps
# copyright 			Copyright (C) 2014 JoomlaThat.com. All Rights Reserved.
# @license				http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: 			http://www.JoomlaThat.com
# Technical Support:	Forum - http://www.JoomlaThat.com/support
-------------------------------------------------------------------------*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.plugin.plugin' );

//new for Joomla 3.0
if(!defined('DS')){
define('DS',DIRECTORY_SEPARATOR);
}

class  plgPayplansInvoicemanager extends XiPlugin
{
	
	public function onPayplansSystemStart()
	{
		$lang = JFactory::getLanguage();
		$lang->load('com_invoices', JPATH_SITE);
		
		return true;
	}
	
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		//print_r($new);die;
		if($this->params->get('action') == "subscription"){

			// no need to trigger if previous and current state is same
			if($prev != null && $prev->getStatus() == $new->getStatus()){
				return true;
			}
	
			$newStatus  = $new->getStatus();
			
			// if subscription is active
			if(true){ //if($newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE){
				
				$data = new stdClass();
			
				$data->reference_id = $new->getId();
				$data->extended_name = $new->getTitle();
				$data->value = $new->getPrice();
				$data->user_id = $new->getBuyer();
				$data->plan_id = $new->reference_id;
				
				$data->name = "PayPlans Subscription";
				$data->type = 1;

				$db = JFactory::getDBO();

				$query = " SELECT inv.invoice_id FROM #__payplans_subscription AS subsc LEFT JOIN #__payplans_invoice AS inv ON subsc.order_id = inv.object_id WHERE subsc.subscription_id = ".$data->reference_id;
				$db->setQuery($query);
				$invoice_id = $db->loadResult();
				//print_r($db);die;
				
				$this->create_invoice($data, $invoice_id, $new);
				
			}
		}
		
		return true;
	}
	
	public function onPayplansInvoiceAfterSave($prev, $new){
		//print_r($new);die;
		if($this->params->get('action') == "invoice"){
		
			$newStatus  = $new->getStatus();
			
			if(true){ //if($newStatus == PayplansStatus::INVOICE_PAID){
				
				$data = new stdClass();
				
				$data->reference_id = $new->getId();
				$data->extended_name = $new->getTitle();
				$data->value = 0;
				$data->user_id = $new->getBuyer();
				$data->plan_id = $new->reference_id;
				
				$data->name = "PayPlans Invoice";
				$data->type = 2;

				$this->create_invoice($data, $data->reference_id, $new);
			
			}
		}
		
		return true;
	}
	
	function create_invoice($data, $pp_invoice_id, $order){
		
		require_once(JPATH_SITE.DS.'components'.DS.'com_invoices'.DS.'helpers'.DS.'helpers.php');
		
		$db = JFactory::getDBO();

		$component = "com_payplans" ;
		
		$is_new = false;
		$must_update = false ;
		$invoice_id = 0 ;

		//we check if this particular order had already been tracked
		$query = " SELECT id FROM #__invoices_invoices WHERE external_type = '".$component."' AND external_id = ".$data->reference_id;
		$db->setQuery($query);
		$exists = $db->loadResult();

		//if it didn't exist, we attemp to create the conversion
		if(!$exists){
			$is_new = true ;
		}
		else{
			$must_update = true ;
			$invoice_id = $exists;
		}

		if($is_new || $must_update){
			
			$params = JComponentHelper::getParams( 'com_invoices' );

			$query = ' SELECT i.real_invoice_num FROM #__invoices_invoices as i WHERE i.type = 1 '.
					 ' ORDER BY i.id DESC ';
			$db->setQuery( $query );
			$real_invoice_num = $db->loadResult() + 1;
			$invoice_num = InvoicesHelper::getInvoiceNum($real_invoice_num) ;

			$user_id = $data->user_id ;
			
			$query = " SELECT *, u.name FROM #__payplans_user AS pu LEFT JOIN #__users AS u ON u.id = pu.user_id WHERE pu.user_id = ".$data->user_id;
			$db->setQuery($query);
			$user = $db->loadObject();
			
			$userdetails = json_decode($user->preference);
			
			$country = "";
			
			if($user->country){
				$query = " SELECT title FROM #__payplans_country WHERE country_id = ".$user->country;
				$db->setQuery($query);
				$country = $db->loadResult();
			}

			$query = " SELECT * FROM #__payplans_invoice WHERE invoice_id = ".$pp_invoice_id;
			$db->setQuery($query);
			$pp_invoice = $db->loadObject();

			if($data->value == 0){
				$data->value = $pp_invoice->subtotal;
			}

			//$order_id = XiHelperUtils::getIdFromKey($data['item_number']);
			$query = " SELECT inv.user_id, modi.* FROM #__payplans_invoice AS inv 
						LEFT JOIN #__payplans_modifier AS modi ON ( modi.invoice_id = inv.invoice_id )
						WHERE inv.invoice_id = ". $pp_invoice_id ;
			$db->setQuery($query);
			$themodifiers = $db->loadObjectList();

			//$tax = array();
			//$discount = array();
			$tax = 0 ;
			$discount = 0 ;

			$itemtax = 0 ;
			$itemdiscount = 0 ;

			foreach($themodifiers as $modifier){
				switch($modifier->type){
					case "eu-vat": case "basictax":
						$tax = $modifier->amount ;
					break;
					case "discount":
						$discount = $modifier->amount * (-1);
					break;
				}
			}

			$taxes = array();

			if($tax){
				$query = " SELECT id FROM #__invoices_taxes WHERE type = 'percent' AND value = '".$tax."' ";
				$db->setQuery($query);
				$storedtax = $db->loadResult();
				if($storedtax) $taxes[] = $storedtax ;
				else $itemtax = $tax ;
			}

			if($discount){
				$itemdiscount = $discount ;
			}
			
			$joomlauser = JFactory::getUser($data->user_id);
			if(!$user->name) $user->name = $joomlauser->name ;
			if(!$user->email) $user->email = $joomlauser->email ;
			
			switch($order->getStatus()){
					
				case 1602; // HOLD
				$status = "pending";
				$payment_status = 0 ;
				break;
				case 1601; // ACTIVE
				$status = "paid";
				$payment_status = 1 ;
				break;
				case 1603; // EXPIRED
				$status = "paid";
				$payment_status = 1 ;
				break;
				case 402; // INVOICE PAID
				$status = "paid";
				$payment_status = 1 ;
				break;
				case 403; // INVOICE REFUNDED
				$status = "refunded";
				$payment_status = 0 ;
				break;
				case 404; // INVOICE RECHARGE
				$status = "refunded";
				$payment_status = 0 ;
				break;
				case 401; // INVOICE CHECKOUT
				$status = "pending";
				$payment_status = 0 ;
				break;
				default:
				$status = $order->getStatus() ;
				$payment_status = 0 ;
				break;
				
			}

			$invoice_data = array(	
								"id" => $invoice_id ,
								"to_name" => $user->name ,
								"to_company" => $userdetails->business_name,
								"to_address" => $user->address,
								"to_email" => $user->email,
								"to_zipcode" => $user->zipcode,
								"to_city" => $user->city,
								"to_state" => $user->state,
								"to_country" => $country,
								"to_phone" => "",
								"to_vatid" => $userdetails->tin,
								"vincular_cliente_checkbox" => 2,
								"from_name" => $params->get('from_name'),
								"from_address" => $params->get('from_address'),
								"from_num" => $params->get('from_num'),
								"from_url" => $params->get('from_url'),
								"from_email" => $params->get('from_email'),
								"from_phone" => $params->get('from_phone'),
								"invoice_num" => $invoice_num,
								"real_invoice_num" => $real_invoice_num,
								"invoice_date" => date('Y-m-d H:i:s'),
								"template_id" => $params->get('invoice_template'),
								"currency_before" => $params->get('currency_before'),
								"currency_after" => $params->get('currency_after'),
								"publish" => $this->params->get('activation'),
								"start_publish" => "",
								"end_publish" => "",
								"joomla_user_id" => $data->user_id,
								"taxes" => $taxes,
								"status" => $status,
								"external_type" => $component,
								"external_id" => $data->reference_id
								) ;
			
			$items = array();
			
			$product_exists = 0 ;

			if($must_update){
				$query = " SELECT id FROM #__invoices_items WHERE invoice_id = '".$invoice_id."' AND external_type = '".$component."' AND product_id = '".$data->plan_id . "' ";
				$db->setQuery($query);
				$product_exists = $db->loadResult();
			}

			$product_id = $product_exists ;

			$items[] = array(
								"id" => $product_id,
								"sku" => $data->plan_id,
								"name" => $data->extended_name,
								"desc" => $data->extended_name,
								"value" => $data->value,
								"discount" => $itemdiscount,
								"amount" => 1,
								"tax" => $itemtax,
								"external_type" => $component,
								"product_id" => $data->plan_id,
								);

			$payments = array();
			
			$payment_exists = 0 ;

			if($must_update){
				$query = " SELECT id FROM #__invoices_payments WHERE invoice_id = '".$invoice_id."' AND payment_type = 'payment_payplans' ";
				$db->setQuery($query);
				$payment_exists = $db->loadResult();
			}
	
			$payment_id = $payment_exists ;
			
			/*
						
			XiParameter Object
			(
				[data:protected] => stdClass Object
					(
						[bankname] => 
						[account_number] => 
						[allow_recurring_cancel] => 0
						[notify_users] => 0
						[from] => Cash
						[id] => 
						[amount] => 12.00
					)
			
				[separator] => .
			)
			*/
			
			$payment_details = array();
			$details = "";
			
			$transactions = $order->getTransactions() ;
			
			if(isset($transactions[1])){
				$transaction = $transactions[1]->getParams() ;
						
				$transaction_id = $transaction->get('id') ;
				
				$payment_details['from'] = $transaction->get('from');
				
			}
			else $transaction_id = "";
			
			foreach($payment_details as $key => $value){
				$details .= $key . ": ". $value ."\n";
			}
			
			$payments[0] = array(
								"id" => $payment_id,
								"payment_details" => $details,
								"payment_datetime" => date('Y-m-d H:i:s'),
								"payment_amount" => $pp_invoice->total,
								"payment_type" => "payment_payplans",
								"payment_description" => "Payment",
								"payment_id" => $transaction_id,
								"payment_status" => $payment_status
								);
							
			InvoicesHelper::create_invoice($invoice_data, $items, $payments);
		}
	}

}

==> download-file?path=joomla%252Finvoicespro330%252Fplugins%252Fplg_invoicemanager_payplans_4.0.2.zip: invoicemanager.php <==
<?php
/*------------------------------------------------------------------------
# com_invoices - Invoice Manager for Joomla
# ------------------------------------------------------------------------
# author				Germinal Camps
# copyright 			Copyright (C) 2014 JoomlaThat.com. All Rights Reserved.
# @license				http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: 			http://www.JoomlaThat.com
# Technical Support:	Forum - http://www.JoomlaThat.com/support
-------------------------------------------------------------------------*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.plugin.plugin' );

//new for Joomla 3.0
if(!defined('DS')){
define('DS',DIRECTORY_SEPARATOR);
}

class  plgPayplansInvoicemanager extends PPPlugins
{

	public function onPayplansSystemStart()
	{
		$lang = JFactory::getLanguage();
		$lang->load('com_invoices', JPATH_SITE);

		return true;
	}

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{

		if($this->params->get('action') == "subscription"){

			// no need to trigger if previous and current state is same
			if($prev != null && $prev->getStatus() == $new->getStatus()){
				return true;
			}

			$newStatus  = $new->getStatus();

			// if subscription is active
			if($newStatus == PP_SUBSCRIPTION_ACTIVE){

				$data = new stdClass();

				$data->reference_id = $new->getId();
				$data->extended_name = $new->getTitle();
				$data->description = $new->getPlan()->description;
				$data->value = $new->getPrice();
				$data->user_id = $new->getBuyer()->id;
				$data->plan_id = $new->table->plan_id;

				$data->name = "PayPlans Subscription";
				$data->type = 1;

				$db = JFactory::getDBO();

				$query = " SELECT inv.invoice_id FROM #__payplans_subscription AS subsc
										LEFT JOIN #__payplans_invoice AS inv ON subsc.order_id = inv.object_id
										WHERE subsc.subscription_id = ".(int)$data->reference_id;
				$db->setQuery($query);
				$invoice_id = (int)$db->loadResult();

				$this->create_invoice($data, $invoice_id, $new);

			}
		}

		return true;
	}

	public function onPayplansInvoiceAfterSave($prev, $new){

		if($this->params->get('action') == "invoice"){

			$newStatus  = $new->getStatus();

			if(true){ //if($newStatus == PP_INVOICE_PAID){

				$data = new stdClass();

				$data->reference_id = $new->getId();
				$data->extended_name = $new->getTitle();
				$data->description = $new->getPlan()->description;
				$data->value = 0;
				$data->user_id = $new->getBuyer()->id;
				$data->plan_id = $new->reference_id;

				$data->name = "PayPlans Invoice";
				$data->type = 2;

				$this->create_invoice($data, $data->reference_id, $new);

			}
		}

		return true;
	}

	function create_invoice($data, $pp_invoice_id, $order){

		require_once(JPATH_SITE.DS.'components'.DS.'com_invoices'.DS.'helpers'.DS.'helpers.php');

		$db = JFactory::getDBO();

		$component = "com_payplans" ;

		$is_new = false;
		$must_update = false ;
		$invoice_id = 0 ;

		//we check if this particular order had already been tracked
		$query = " SELECT id FROM #__invoices_invoices WHERE external_type = '".$component."' AND external_id = ".$data->reference_id;
		$db->setQuery($query);
		$exists = $db->loadResult();

		//if it didn't exist, we attemp to create the conversion
		if(!$exists){
			$is_new = true ;
		}
		else{
			$must_update = true ;
			$invoice_id = $exists;
		}

		if($is_new || $must_update){

			$params = JComponentHelper::getParams( 'com_invoices' );

			$query = ' SELECT i.real_invoice_num FROM #__invoices_invoices as i WHERE i.type = 1 '.
					 ' ORDER BY i.id DESC ';
			$db->setQuery( $query );
			$real_invoice_num = $db->loadResult() + 1;
			$invoice_num = InvoicesHelper::getInvoiceNum($real_invoice_num) ;

			$user_id = $data->user_id ;

			$query = " SELECT *, u.name FROM #__payplans_user AS pu LEFT JOIN #__users AS u ON u.id = pu.user_id WHERE pu.user_id = ".$user_id;
			$db->setQuery($query);
			$user = $db->loadObject();

			$userdetails = json_decode($user->params);

			$country = "";

			if($user->country){
				$query = " SELECT title FROM #__payplans_country WHERE country_id = ".(int)$user->country;
				$db->setQuery($query);
				$country = $db->loadResult();
			}

			$query = " SELECT * FROM #__payplans_invoice WHERE invoice_id = ".(int)$pp_invoice_id;
			$db->setQuery($query);
			$pp_invoice = $db->loadObject();

			if($data->value == 0){
				$data->value = $pp_invoice->subtotal;
			}

			$query = " SELECT inv.user_id, modi.* FROM #__payplans_invoice AS inv
						LEFT JOIN #__payplans_modifier AS modi ON ( modi.invoice_id = inv.invoice_id )
						WHERE inv.invoice_id = ". $pp_invoice_id ;
			$db->setQuery($query);
			$themodifiers = $db->loadObjectList();

			$tax = 0 ;
			$discount = 0 ;

			$itemtax = 0 ;
			$itemdiscount = 0 ;

			foreach($themodifiers as $modifier){
				switch($modifier->type){
					case "eu-vat": case "basictax":
						$tax = $modifier->amount ;
					break;
					case "discount":
						$discount = $modifier->amount * (-1);
					break;
				}
			}

			$taxes = array();

			if($tax){
				$query = " SELECT id FROM #__invoices_taxes WHERE type = 'percent' AND value = '".$tax."' ";
				$db->setQuery($query);
				$storedtax = $db->loadResult();
				if($storedtax) $taxes[] = $storedtax ;
				else $itemtax = $tax ;
			}

			if($discount){
				$itemdiscount = $discount ;
			}

			$joomlauser = JFactory::getUser($data->user_id);
			if(!$user->name) $user->name = $joomlauser->name ;
			if(!$user->email) $user->email = $joomlauser->email ;

			switch($order->getStatus()){

				case 1602; // HOLD
				$status = "pending";
				$payment_status = 0 ;
				break;
				case 1601; // ACTIVE
				$status = "paid";
				$payment_status = 1 ;
				break;
				case 1603; // EXPIRED
				$status = "paid";
				$payment_status = 1 ;
				break;
				case 402; // INVOICE PAID
				$status = "paid";
				$payment_status = 1 ;
				break;
				case 403; // INVOICE REFUNDED
				$status = "refunded";
				$payment_status = 0 ;
				break;
				case 404; // INVOICE RECHARGE
				$status = "refunded";
				$payment_status = 0 ;
				break;
				case 401; // INVOICE CHECKOUT
				$status = "pending";
				$payment_status = 0 ;
				break;
				default:
				$status = $order->getStatus() ;
				$payment_status = 0 ;
				break;

			}

			$invoice_data = array(
								"id" => $invoice_id ,
								"to_name" => $user->name ,
								"to_company" => $userdetails->business_name,
								"to_address" => $user->address,
								"to_email" => $user->email,
								"to_zipcode" => $user->zipcode,
								"to_city" => $user->city,
								"to_state" => $user->state,
								"to_country" => $country,
								"to_phone" => "",
								"to_vatid" => $userdetails->tin,
								"vincular_cliente_checkbox" => 2,
								"from_name" => $params->get('from_name'),
								"from_address" => $params->get('from_address'),
								"from_num" => $params->get('from_num'),
								"from_url" => $params->get('from_url'),
								"from_email" => $params->get('from_email'),
								"from_phone" => $params->get('from_phone'),
								"invoice_num" => $invoice_num,
								"real_invoice_num" => $real_invoice_num,
								"invoice_date" => date('Y-m-d H:i:s'),
								"template_id" => $params->get('invoice_template'),
								"currency_before" => $params->get('currency_before'),
								"currency_after" => $params->get('currency_after'),
								"publish" => $this->params->get('activation'),
								"start_publish" => "",
								"end_publish" => "",
								"joomla_user_id" => $data->user_id,
								"taxes" => $taxes,
								"status" => $status,
								"external_type" => $component,
								"external_id" => $data->reference_id
								) ;

			$items = array();

			$product_exists = 0 ;

			if($must_update){
				$query = " SELECT id FROM #__invoices_items WHERE invoice_id = '".$invoice_id."' AND external_type = '".$component."' AND product_id = '".$data->plan_id . "' ";
				$db->setQuery($query);
				$product_exists = $db->loadResult();
			}

			$product_id = $product_exists ;

			$items[] = array(
								"id" => $product_id,
								"sku" => $data->plan_id,
								"name" => $data->extended_name,
								"desc" => strip_tags($data->description),
								"value" => $data->value,
								"discount" => $itemdiscount,
								"amount" => 1,
								"tax" => $itemtax,
								"external_type" => $component,
								"product_id" => $data->plan_id,
								);

			$payments = array();

			$payment_exists = 0 ;

			if($must_update){
				$query = " SELECT id FROM #__invoices_payments WHERE invoice_id = '".$invoice_id."' AND payment_type = 'payment_payplans' ";
				$db->setQuery($query);
				$payment_exists = $db->loadResult();
			}

			$payment_id = $payment_exists ;

			$payment_details = array();
			$details = "";

			$transactions = $order->getOrder()->getInvoice()->getTransactions() ;
			$payment = $order->getOrder()->getInvoice()->getPayment() ;

			if(isset($transactions[0])){
				$transaction = $transactions[0];

				// print_r(get_class_methods($transaction) );
				// print_r($transaction);
				// print_r(get_class_methods($payment) );
				// print_r($payment);
				// print_r($payment->getAppName());
				// print_r($payment->getApp());
				// print_r($transaction->getGatewayTxnId());
				// die;

				$transaction_id = $transaction->getGatewayTxnId() ;

				$payment_details['transaction_id'] = $transaction_id;

			}
			else $transaction_id = "";

			$transaction_id = "";

			foreach($payment_details as $key => $value){
				$details .= $key . ": ". $value ."\n";
			}

			$payments[0] = array(
								"id" => $payment_id,
								"payment_details" => $details,
								"payment_datetime" => date('Y-m-d H:i:s'),
								"payment_amount" => $pp_invoice->total,
								"payment_type" => $payment ? $payment->getAppName() : '',
								"payment_description" => "Payment",
								"payment_id" => $transaction_id,
								"payment_status" => $payment_status
								);

			InvoicesHelper::create_invoice($invoice_data, $items, $payments);
		}
	}

}
